// Yylex.lex -*- java -*- lexical analyser for ACAP

package acap;

import java.io.DataOutputStream;
import java.util.Hashtable;
import java.util.Vector;

class Yytoken {
  static final int EOF = 0;
  static final int EOL = 1;
  static final int TAG = 2;
  static final int STRING = 3;
  static final int INT = 4;
  static final int STATUS = 5;
  static final int ACAP = 6;
  static final int ADDTO = 7;
  static final int REMOVEFROM = 8;
  static final int CHANGE = 9;
  static final int ENTRY = 10;
  static final int MODTIME = 11;
  static final int OK = 12;
  static final int NO = 13;
  static final int BAD = 14;
  static final int BYE = 15;
  private int token;
  private String str;
  private int ival;
  public Yytoken(int token, String str) {
    this.token = token;
    this.str = str;
  }
  public Yytoken(int token, int ival) {
    this.token = token;
    this.ival = ival;
  }
  public Yytoken(int token) {
    this(token, null);
  }
  public String Str() {
    return str;
  }
  public int Int() {
    return ival;
  }
  public void Assert(int token) throws LexerException {
    if (this.token != token) {
      throw new LexerException();
    }
  }
  public int Tok() {
    return token;
  }
}

class LexerException extends Exception {
  public LexerException() {
    super();
  }
}

%%

%notunix
%full
%{
		// acap-specific lexer state
		StringBuffer literal = null;
		int litcount = 0;

		public void Resync() {
		  if (yy_lexical_state != TAG) {
		    yybegin(ERR);
		  }
		}
%}
%init{
		// acap-specific lexer initialization
		yybegin(TAG);
%init}

%yylexthrow{
		       LexerException
%yylexthrow}

TAG_CHAR=[^\(\)\{\%\* \x00-\x1f\x7f-\xff\"\\\+]
ATOM_CHAR=[^\(\)\{\%\* \x00-\x1f\x7f-\xff\"\\]
TEXT_CHAR=[^\r\n\"\\\x80-\xff]
QUOTED_SPECIALS=[\"\\]

%state TAG
%state WS
%state ERR
%state N
%state LDAT
%state LINE
%state LITERAL

%%

<TAG>[\.a-zA-Z0-9]+				{ yybegin(WS); return new Yytoken(Yytoken.TAG, yytext()); }
<TAG>"*"					{ yybegin(WS); return new Yytoken(Yytoken.TAG, yytext()); }

<WS>" "						{ yybegin(N); }
<WS>\r						{ yybegin(TAG); return new Yytoken(Yytoken.EOL); }

<ERR>\r						{ yybegin(TAG); throw new LexerException(); }
<ERR>[^\r]					{}

<LINE>" "					{ yybegin(LDAT); }

<LDAT>[^\r]*\r					{ yybegin(TAG); return new Yytoken(Yytoken.STATUS, yytext()); }

<LITERAL>.|\r|\n				{
  literal.append(yytext().charAt(0));
  if (--litcount == 0) {
    yybegin(WS);
    String res = new String(literal);
    literal = null;
    return new Yytoken(Yytoken.STRING, res);
  }
						}

<N>[Aa][Cc][Aa][Pp]				{ yybegin(WS); return new Yytoken(Yytoken.ACAP); }
<N>[Aa][Dd][Dd][Tt][Oo]				{ yybegin(WS); return new Yytoken(Yytoken.ADDTO); }
<N>[Rr][Ee][Mm][Oo][Vv][Ee][Ff][Rr][Oo][Mm]	{ yybegin(WS); return new Yytoken(Yytoken.REMOVEFROM); }
<N>[Cc][Hh][Aa][Nn][Gg][Ee]			{ yybegin(WS); return new Yytoken(Yytoken.CHANGE); }
<N>[Ee][Nn][Tt][Rr][Yy]				{ yybegin(WS); return new Yytoken(Yytoken.ENTRY); }
<N>[Mm][Oo][Dd][Tt][Ii][Mm][Ee]			{ yybegin(WS); return new Yytoken(Yytoken.MODTIME); }
<N>[Oo][Kk]					{ yybegin(LINE); return new Yytoken(Yytoken.OK); }
<N>[Nn][Oo]					{ yybegin(LINE); return new Yytoken(Yytoken.NO); }
<N>[Bb][Aa][Dd]					{ yybegin(LINE); return new Yytoken(Yytoken.BAD); }
<N>[Bb][Yy][Ee]					{ yybegin(LINE); return new Yytoken(Yytoken.BYE); }
<N>\"({TEXT_CHAR}|\\{QUOTED_SPECIALS})*\"	{ yybegin(WS); return new Yytoken(Yytoken.STRING, yytext().substring(1, yylength() - 2)); }
<N>[0-9]+					{ yybegin(WS); return new Yytoken(Yytoken.INT, Integer.parseInt(yytext())); }
<N>\{[0-9]+\}\r\n				{
						  // Wheee!  It's a LITERAL!
  litcount = Integer.parseInt(yytext().substring(1, yylength() - 3));
  if (litcount > 0) {
    literal = new StringBuffer();
    literal.ensureCapacity(litcount);
    yybegin(LITERAL);
  } else {
    yybegin(WS);
    return new Yytoken(Yytoken.STRING, "");
  }
						}
\n						{}
. 						{ yybegin(ERR); }
